@@ -29,7 +29,7 @@ Options:
     --locked                 Require Cargo.lock is up to date
 ";
 
-pub fn execute(options: Options, config: &Config) -> CliResult<Option<()>> {
+pub fn execute(options: Options, config: &Config) -> CliResult {
     config.configure(options.flag_verbose,
                      options.flag_quiet,
                      &options.flag_color,
@@ -46,5 +46,5 @@ pub fn execute(options: Options, config: &Config) -> CliResult<Option<()>> {
 
     source.update()?;
 
-    Ok(None)
+    Ok(())
 }
