@@ -1,3 +1,4 @@
+use cargo;
 use cargo::util::{CliResult, CliError, human, ChainError, Config};
 use cargo::util::important_paths::{find_root_manifest_for_wd};
 
@@ -23,14 +24,16 @@ pub struct ProjectLocation {
 }
 
 pub fn execute(flags: LocateProjectFlags,
-               config: &Config) -> CliResult<Option<ProjectLocation>> {
-    let root = try!(find_root_manifest_for_wd(flags.flag_manifest_path, config.cwd()));
+               config: &Config) -> CliResult {
+    let root = find_root_manifest_for_wd(flags.flag_manifest_path, config.cwd())?;
 
-    let string = try!(root.to_str()
+    let string = root.to_str()
                       .chain_error(|| human("Your project path contains \
                                              characters not representable in \
                                              Unicode"))
-                      .map_err(|e| CliError::from_boxed(e, 1)));
+                      .map_err(|e| CliError::new(e, 1))?;
 
-    Ok(Some(ProjectLocation { root: string.to_string() }))
+    let location = ProjectLocation { root: string.to_string() };
+    cargo::print_json(&location);
+    Ok(())
 }
