@@ -34,7 +34,7 @@ Options:
 
 ";
 
-pub fn execute(options: Options, config: &Config) -> CliResult<Option<()>> {
+pub fn execute(options: Options, config: &Config) -> CliResult {
     config.configure(options.flag_verbose,
                      options.flag_quiet,
                      &options.flag_color,
@@ -60,6 +60,6 @@ pub fn execute(options: Options, config: &Config) -> CliResult<Option<()>> {
 
     let token = token.trim().to_string();
     ops::registry_login(config, token)?;
-    Ok(None)
+    Ok(())
 }
 
