@@ -1,3 +1,4 @@
+use std::collections::HashMap;
 use std::fmt;
 use std::path::{PathBuf, Path};
 
@@ -47,6 +48,7 @@ pub struct VirtualManifest {
 pub struct ManifestMetadata {
     pub authors: Vec<String>,
     pub keywords: Vec<String>,
+    pub categories: Vec<String>,
     pub license: Option<String>,
     pub license_file: Option<String>,
     pub description: Option<String>,    // not markdown
@@ -54,6 +56,7 @@ pub struct ManifestMetadata {
     pub homepage: Option<String>,       // url
     pub repository: Option<String>,     // url
     pub documentation: Option<String>,  // url
+    pub badges: HashMap<String, HashMap<String, String>>,
 }
 
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
@@ -104,18 +107,20 @@ pub enum TargetKind {
     Bin,
     Test,
     Bench,
-    Example,
+    ExampleLib(Vec<LibKind>),
+    ExampleBin,
     CustomBuild,
 }
 
 impl Encodable for TargetKind {
     fn encode<S: Encoder>(&self, s: &mut S) -> Result<(), S::Error> {
         match *self {
-            TargetKind::Lib(ref kinds) => {
-                kinds.iter().map(|k| k.crate_type()).collect()
+            TargetKind::Lib(ref kinds) |
+            TargetKind::ExampleLib(ref kinds) => {
+                kinds.iter().map(LibKind::crate_type).collect()
             }
             TargetKind::Bin => vec!["bin"],
-            TargetKind::Example => vec!["example"],
+            TargetKind::ExampleBin => vec!["example"],
             TargetKind::Test => vec!["test"],
             TargetKind::CustomBuild => vec!["custom-build"],
             TargetKind::Bench => vec!["bench"],
@@ -130,7 +135,7 @@ pub struct Profile {
     pub codegen_units: Option<u32>,    // None = use rustc default
     pub rustc_args: Option<Vec<String>>,
     pub rustdoc_args: Option<Vec<String>>,
-    pub debuginfo: bool,
+    pub debuginfo: Option<u32>,
     pub debug_assertions: bool,
     pub rpath: bool,
     pub test: bool,
@@ -143,7 +148,7 @@ pub struct Profile {
 #[derive(RustcEncodable)]
 struct SerializedProfile<'a> {
     opt_level: &'a str,
-    debuginfo: bool,
+    debuginfo: Option<u32>,
     debug_assertions: bool,
     test: bool,
 }
@@ -170,6 +175,7 @@ pub struct Profiles {
     pub doc: Profile,
     pub custom_build: Profile,
     pub check: Profile,
+    pub doctest: Profile,
 }
 
 /// Information about a binary, a library, an example, etc. that is part of the
@@ -293,11 +299,12 @@ impl VirtualManifest {
 }
 
 impl Target {
-    fn blank() -> Target {
+    fn with_path(src_path: PathBuf) -> Target {
+        assert!(src_path.is_absolute());
         Target {
             kind: TargetKind::Bin,
             name: String::new(),
-            src_path: PathBuf::new(),
+            src_path: src_path,
             doc: false,
             doctest: false,
             harness: true,
@@ -309,67 +316,69 @@ impl Target {
 
     pub fn lib_target(name: &str,
                       crate_targets: Vec<LibKind>,
-                      src_path: &Path) -> Target {
+                      src_path: PathBuf) -> Target {
         Target {
             kind: TargetKind::Lib(crate_targets),
             name: name.to_string(),
-            src_path: src_path.to_path_buf(),
             doctest: true,
             doc: true,
-            ..Target::blank()
+            ..Target::with_path(src_path)
         }
     }
 
-    pub fn bin_target(name: &str, src_path: &Path) -> Target {
+    pub fn bin_target(name: &str, src_path: PathBuf) -> Target {
         Target {
             kind: TargetKind::Bin,
             name: name.to_string(),
-            src_path: src_path.to_path_buf(),
             doc: true,
-            ..Target::blank()
+            ..Target::with_path(src_path)
         }
     }
 
     /// Builds a `Target` corresponding to the `build = "build.rs"` entry.
-    pub fn custom_build_target(name: &str, src_path: &Path) -> Target {
+    pub fn custom_build_target(name: &str, src_path: PathBuf) -> Target {
         Target {
             kind: TargetKind::CustomBuild,
             name: name.to_string(),
-            src_path: src_path.to_path_buf(),
             for_host: true,
             benched: false,
             tested: false,
-            ..Target::blank()
+            ..Target::with_path(src_path)
         }
     }
 
-    pub fn example_target(name: &str, src_path: &Path) -> Target {
+    pub fn example_target(name: &str,
+                          crate_targets: Vec<LibKind>,
+                          src_path: PathBuf) -> Target {
+        let kind = if crate_targets.is_empty() {
+            TargetKind::ExampleBin
+        } else {
+            TargetKind::ExampleLib(crate_targets)
+        };
+
         Target {
-            kind: TargetKind::Example,
+            kind: kind,
             name: name.to_string(),
-            src_path: src_path.to_path_buf(),
             benched: false,
-            ..Target::blank()
+            ..Target::with_path(src_path)
         }
     }
 
-    pub fn test_target(name: &str, src_path: &Path) -> Target {
+    pub fn test_target(name: &str, src_path: PathBuf) -> Target {
         Target {
             kind: TargetKind::Test,
             name: name.to_string(),
-            src_path: src_path.to_path_buf(),
             benched: false,
-            ..Target::blank()
+            ..Target::with_path(src_path)
         }
     }
 
-    pub fn bench_target(name: &str, src_path: &Path) -> Target {
+    pub fn bench_target(name: &str, src_path: PathBuf) -> Target {
         Target {
             kind: TargetKind::Bench,
             name: name.to_string(),
-            src_path: src_path.to_path_buf(),
             tested: false,
-            ..Target::blank()
+            ..Target::with_path(src_path)
         }
     }
 
@@ -386,7 +395,11 @@ impl Target {
     pub fn doctested(&self) -> bool {
         self.doctest && match self.kind {
             TargetKind::Lib(ref kinds) => {
-                kinds.contains(&LibKind::Rlib) || kinds.contains(&LibKind::Lib)
+                kinds.iter().find(|k| {
+                  *k == &LibKind::Rlib ||
+                  *k == &LibKind::Lib ||
+                  *k == &LibKind::ProcMacro
+                }).is_some()
             }
             _ => false,
         }
@@ -420,7 +433,15 @@ impl Target {
     }
 
     pub fn is_bin(&self) -> bool { self.kind == TargetKind::Bin }
-    pub fn is_example(&self) -> bool { self.kind == TargetKind::Example }
+
+    pub fn is_example(&self) -> bool {
+        match self.kind {
+            TargetKind::ExampleBin |
+            TargetKind::ExampleLib(..) => true,
+            _ => false
+        }
+    }
+
     pub fn is_test(&self) -> bool { self.kind == TargetKind::Test }
     pub fn is_bench(&self) -> bool { self.kind == TargetKind::Bench }
     pub fn is_custom_build(&self) -> bool { self.kind == TargetKind::CustomBuild }
@@ -428,13 +449,14 @@ impl Target {
     /// Returns the arguments suitable for `--crate-type` to pass to rustc.
     pub fn rustc_crate_types(&self) -> Vec<&str> {
         match self.kind {
-            TargetKind::Lib(ref kinds) => {
-                kinds.iter().map(|kind| kind.crate_type()).collect()
-            },
+            TargetKind::Lib(ref kinds) |
+            TargetKind::ExampleLib(ref kinds) => {
+                kinds.iter().map(LibKind::crate_type).collect()
+            }
             TargetKind::CustomBuild |
             TargetKind::Bench |
             TargetKind::Test |
-            TargetKind::Example |
+            TargetKind::ExampleBin |
             TargetKind::Bin => vec!["bin"],
         }
     }
@@ -483,7 +505,8 @@ impl fmt::Display for Target {
             TargetKind::Bin => write!(f, "Target(bin: {})", self.name),
             TargetKind::Test => write!(f, "Target(test: {})", self.name),
             TargetKind::Bench => write!(f, "Target(bench: {})", self.name),
-            TargetKind::Example => write!(f, "Target(example: {})", self.name),
+            TargetKind::ExampleBin |
+            TargetKind::ExampleLib(..) => write!(f, "Target(example: {})", self.name),
             TargetKind::CustomBuild => write!(f, "Target(script)"),
         }
     }
@@ -492,7 +515,7 @@ impl fmt::Display for Target {
 impl Profile {
     pub fn default_dev() -> Profile {
         Profile {
-            debuginfo: true,
+            debuginfo: Some(2),
             debug_assertions: true,
             ..Profile::default()
         }
@@ -501,7 +524,7 @@ impl Profile {
     pub fn default_release() -> Profile {
         Profile {
             opt_level: "3".to_string(),
-            debuginfo: false,
+            debuginfo: None,
             ..Profile::default()
         }
     }
@@ -540,6 +563,14 @@ impl Profile {
             ..Profile::default_dev()
         }
     }
+
+    pub fn default_doctest() -> Profile {
+        Profile {
+            doc: true,
+            test: true,
+            ..Profile::default_dev()
+        }
+    }
 }
 
 impl Default for Profile {
@@ -550,7 +581,7 @@ impl Default for Profile {
             codegen_units: None,
             rustc_args: None,
             rustdoc_args: None,
-            debuginfo: false,
+            debuginfo: None,
             debug_assertions: false,
             rpath: false,
             test: false,
