@@ -6,6 +6,7 @@ use std::fs::{self, File};
 use std::io::prelude::*;
 use std::io::SeekFrom;
 use std::path::{Path, PathBuf};
+use std::sync::Arc;
 
 use semver::Version;
 use tempdir::TempDir;
@@ -13,7 +14,7 @@ use toml;
 
 use core::{SourceId, Source, Package, Dependency, PackageIdSpec};
 use core::{PackageId, Workspace};
-use ops::{self, CompileFilter};
+use ops::{self, CompileFilter, DefaultExecutor};
 use sources::{GitSource, PathSource, SourceConfigMap};
 use util::{CargoResult, ChainError, Config, human, internal};
 use util::{Filesystem, FileLock};
@@ -96,6 +97,8 @@ pub fn install(root: Option<&str>,
     };
     let pkg = ws.current()?;
 
+    config.shell().status("Installing", pkg)?;
+
     // Preflight checks to check up front whether we'll overwrite something.
     // We have to check this again afterwards, but may as well avoid building
     // anything if we're gonna throw it away anyway.
@@ -106,7 +109,10 @@ pub fn install(root: Option<&str>,
         check_overwrites(&dst, pkg, &opts.filter, &list, force)?;
     }
 
-    let compile = ops::compile_ws(&ws, Some(source), opts).chain_error(|| {
+    let compile = ops::compile_ws(&ws,
+                                  Some(source),
+                                  opts,
+                                  Arc::new(DefaultExecutor)).chain_error(|| {
         if let Some(td) = td_opt.take() {
             // preserve the temporary directory, so the user can inspect it
             td.into_path();
