@@ -9,6 +9,7 @@ use git2::Config as GitConfig;
 
 use term::color::BLACK;
 
+use core::Workspace;
 use util::{GitRepo, HgRepo, CargoResult, human, ChainError, internal};
 use util::{Config, paths};
 
@@ -20,6 +21,7 @@ pub enum VersionControl { Git, Hg, NoVcs }
 pub struct NewOptions<'a> {
     pub version_control: Option<VersionControl>,
     pub bin: bool,
+    pub lib: bool,
     pub path: &'a str,
     pub name: Option<&'a str>,
 }
@@ -40,7 +42,7 @@ struct MkOptions<'a> {
 
 impl Decodable for VersionControl {
     fn decode<D: Decoder>(d: &mut D) -> Result<VersionControl, D::Error> {
-        Ok(match &try!(d.read_str())[..] {
+        Ok(match &d.read_str()?[..] {
             "git" => VersionControl::Git,
             "hg" => VersionControl::Hg,
             "none" => VersionControl::NoVcs,
@@ -52,6 +54,31 @@ impl Decodable for VersionControl {
     }
 }
 
+impl<'a> NewOptions<'a> {
+    pub fn new(version_control: Option<VersionControl>,
+           bin: bool,
+           lib: bool,
+           path: &'a str,
+           name: Option<&'a str>) -> NewOptions<'a> {
+
+        // default to lib
+        let is_lib = if !bin {
+            true
+        }
+        else {
+            lib
+        };
+
+        NewOptions {
+            version_control: version_control,
+            bin: bin,
+            lib: is_lib,
+            path: path,
+            name: name,
+        }
+    }
+}
+
 struct CargoNewConfig {
     name: Option<String>,
     email: Option<String>,
@@ -68,10 +95,10 @@ fn get_name<'a>(path: &'a Path, opts: &'a NewOptions, config: &Config) -> CargoR
                               path.as_os_str());
     }
 
-    let dir_name = try!(path.file_name().and_then(|s| s.to_str()).chain_error(|| {
+    let dir_name = path.file_name().and_then(|s| s.to_str()).chain_error(|| {
         human(&format!("cannot create a project with a non-unicode name: {:?}",
                        path.file_name().unwrap()))
-    }));
+    })?;
 
     if opts.bin {
         Ok(dir_name)
@@ -81,7 +108,7 @@ fn get_name<'a>(path: &'a Path, opts: &'a NewOptions, config: &Config) -> CargoR
             let message = format!(
                 "note: package will be named `{}`; use --name to override",
                 new_name);
-            try!(config.shell().say(&message, BLACK));
+            config.shell().say(&message, BLACK)?;
         }
         Ok(new_name)
     }
@@ -108,6 +135,13 @@ fn check_name(name: &str) -> CargoResult<()> {
                name)
     }
 
+    if let Some(ref c) = name.chars().nth(0) {
+        if c.is_digit(10) {
+            bail!("Package names starting with a digit cannot be used as a crate name\n\
+               use --name to override crate name")
+        }
+    }
+
     for c in name.chars() {
         if c.is_alphanumeric() { continue }
         if c == '_' || c == '-' { continue }
@@ -169,7 +203,7 @@ fn detect_source_paths_and_types(project_path : &Path,
                 }
             }
             H::Detect => {
-                let content = try!(paths::read(&path.join(pp.clone())));
+                let content = paths::read(&path.join(pp.clone()))?;
                 let isbin = content.contains("fn main");
                 SourceFileInformation {
                     relative_path: pp,
@@ -234,8 +268,12 @@ pub fn new(opts: NewOptions, config: &Config) -> CargoResult<()> {
               path.display())
     }
 
-    let name = try!(get_name(&path, &opts, config));
-    try!(check_name(name));
+    if opts.lib && opts.bin {
+        bail!("can't specify both lib and binary outputs");
+    }
+
+    let name = get_name(&path, &opts, config)?;
+    check_name(name)?;
 
     let mkopts = MkOptions {
         version_control: opts.version_control,
@@ -259,12 +297,16 @@ pub fn init(opts: NewOptions, config: &Config) -> CargoResult<()> {
         bail!("`cargo init` cannot be run on existing Cargo projects")
     }
 
-    let name = try!(get_name(&path, &opts, config));
-    try!(check_name(name));
+    if opts.lib && opts.bin {
+        bail!("can't specify both lib and binary outputs");
+    }
+
+    let name = get_name(&path, &opts, config)?;
+    check_name(name)?;
 
     let mut src_paths_types = vec![];
 
-    try!(detect_source_paths_and_types(&path, name, &mut src_paths_types));
+    detect_source_paths_and_types(&path, name, &mut src_paths_types)?;
 
     if src_paths_types.len() == 0 {
         src_paths_types.push(plan_new_source_file(opts.bin, name.to_string()));
@@ -334,7 +376,7 @@ fn existing_vcs_repo(path: &Path, cwd: &Path) -> bool {
 fn mk(config: &Config, opts: &MkOptions) -> CargoResult<()> {
     let path = opts.path;
     let name = opts.name;
-    let cfg = try!(global_config(config));
+    let cfg = global_config(config)?;
     let mut ignore = "target\n".to_string();
     let in_existing_vcs_repo = existing_vcs_repo(path.parent().unwrap(), config.cwd());
     if !opts.bin {
@@ -351,22 +393,22 @@ fn mk(config: &Config, opts: &MkOptions) -> CargoResult<()> {
     match vcs {
         VersionControl::Git => {
             if !fs::metadata(&path.join(".git")).is_ok() {
-                try!(GitRepo::init(path, config.cwd()));
+                GitRepo::init(path, config.cwd())?;
             }
-            try!(paths::append(&path.join(".gitignore"), ignore.as_bytes()));
+            paths::append(&path.join(".gitignore"), ignore.as_bytes())?;
         },
         VersionControl::Hg => {
             if !fs::metadata(&path.join(".hg")).is_ok() {
-                try!(HgRepo::init(path, config.cwd()));
+                HgRepo::init(path, config.cwd())?;
             }
-            try!(paths::append(&path.join(".hgignore"), ignore.as_bytes()));
+            paths::append(&path.join(".hgignore"), ignore.as_bytes())?;
         },
         VersionControl::NoVcs => {
-            try!(fs::create_dir_all(path));
+            fs::create_dir_all(path)?;
         },
     };
 
-    let (author_name, email) = try!(discover_author());
+    let (author_name, email) = discover_author()?;
     // Hoo boy, sure glad we've got exhaustivenes checking behind us.
     let author = match (cfg.name, cfg.email, author_name, email) {
         (Some(name), Some(email), _, _) |
@@ -403,14 +445,14 @@ path = {}
 
     // Create Cargo.toml file with necessary [lib] and [[bin]] sections, if needed
 
-    try!(paths::write(&path.join("Cargo.toml"), format!(
+    paths::write(&path.join("Cargo.toml"), format!(
 r#"[package]
 name = "{}"
 version = "0.1.0"
 authors = [{}]
 
 [dependencies]
-{}"#, name, toml::Value::String(author), cargotoml_path_specifier).as_bytes()));
+{}"#, name, toml::Value::String(author), cargotoml_path_specifier).as_bytes())?;
 
 
     // Create all specified source files
@@ -421,7 +463,7 @@ authors = [{}]
         let path_of_source_file = path.join(i.relative_path.clone());
 
         if let Some(src_dir) = path_of_source_file.parent() {
-            try!(fs::create_dir_all(src_dir));
+            fs::create_dir_all(src_dir)?;
         }
 
         let default_file_content : &[u8] = if i.bin {
@@ -442,10 +484,16 @@ mod tests {
         };
 
         if !fs::metadata(&path_of_source_file).map(|x| x.is_file()).unwrap_or(false) {
-            return paths::write(&path_of_source_file, default_file_content)
+            paths::write(&path_of_source_file, default_file_content)?;
         }
     }
 
+    if let Err(e) = Workspace::new(&path.join("Cargo.toml"), config) {
+        let msg = format!("compiling this new crate may not work due to invalid \
+                           workspace configuration\n\n{}", e);
+        config.shell().warn(msg)?;
+    }
+
     Ok(())
 }
 
@@ -485,9 +533,9 @@ fn discover_author() -> CargoResult<(String, Option<String>)> {
 }
 
 fn global_config(config: &Config) -> CargoResult<CargoNewConfig> {
-    let name = try!(config.get_string("cargo-new.name")).map(|s| s.val);
-    let email = try!(config.get_string("cargo-new.email")).map(|s| s.val);
-    let vcs = try!(config.get_string("cargo-new.vcs"));
+    let name = config.get_string("cargo-new.name")?.map(|s| s.val);
+    let email = config.get_string("cargo-new.email")?.map(|s| s.val);
+    let vcs = config.get_string("cargo-new.vcs")?;
 
     let vcs = match vcs.as_ref().map(|p| (&p.val[..], &p.definition)) {
         Some(("git", _)) => Some(VersionControl::Git),
