@@ -48,6 +48,7 @@ pub struct Context<'a, 'cfg: 'a> {
     target_info: TargetInfo,
     host_info: TargetInfo,
     profiles: &'a Profiles,
+    incremental_enabled: bool,
 }
 
 #[derive(Clone, Default)]
@@ -74,6 +75,11 @@ impl<'a, 'cfg> Context<'a, 'cfg> {
             None => None,
         };
 
+        // Enable incremental builds if the user opts in. For now,
+        // this is an environment variable until things stabilize a
+        // bit more.
+        let incremental_enabled = env::var("CARGO_INCREMENTAL").is_ok();
+
         Ok(Context {
             ws: ws,
             host: host_layout,
@@ -93,6 +99,7 @@ impl<'a, 'cfg> Context<'a, 'cfg> {
             build_explicit_deps: HashMap::new(),
             links: Links::new(),
             used_in_plugin: HashSet::new(),
+            incremental_enabled: incremental_enabled,
         })
     }
 
@@ -133,6 +140,7 @@ impl<'a, 'cfg> Context<'a, 'cfg> {
         for unit in units {
             self.visit_crate_type(unit, &mut crate_types)?;
         }
+        debug!("probe_target_info: crate_types={:?}", crate_types);
         self.probe_target_info_kind(&crate_types, Kind::Target)?;
         if self.requested_target().is_none() {
             self.host_info = self.target_info.clone();
@@ -146,9 +154,6 @@ impl<'a, 'cfg> Context<'a, 'cfg> {
                         unit: &Unit<'a>,
                         crate_types: &mut BTreeSet<String>)
                         -> CargoResult<()> {
-        if unit.profile.check {
-            crate_types.insert("metadata".to_string());
-        }
         for target in unit.pkg.manifest().targets() {
             crate_types.extend(target.rustc_crate_types().iter().map(|s| {
                 if *s == "lib" {
@@ -169,9 +174,9 @@ impl<'a, 'cfg> Context<'a, 'cfg> {
                               kind: Kind)
                               -> CargoResult<()> {
         let rustflags = env_args(self.config,
-                                      &self.build_config,
-                                      kind,
-                                      "RUSTFLAGS")?;
+                                 &self.build_config,
+                                 kind,
+                                 "RUSTFLAGS")?;
         let mut process = self.config.rustc()?.process();
         process.arg("-")
                .arg("--crate-name").arg("___")
@@ -180,11 +185,7 @@ impl<'a, 'cfg> Context<'a, 'cfg> {
                .env_remove("RUST_LOG");
 
         for crate_type in crate_types {
-            // Here and below we'll skip the metadata crate-type because it is
-            // not supported by older compilers. We'll do this one manually.
-            if crate_type != "metadata" {
-                process.arg("--crate-type").arg(crate_type);
-            }
+            process.arg("--crate-type").arg(crate_type);
         }
         if kind == Kind::Target {
             process.arg("--target").arg(&self.target_triple());
@@ -216,9 +217,6 @@ impl<'a, 'cfg> Context<'a, 'cfg> {
                 map.insert(crate_type.to_string(), None);
                 continue;
             }
-            if crate_type == "metadata" {
-                continue;
-            }
             let line = match lines.next() {
                 Some(line) => line,
                 None => bail!("malformed output when learning about \
@@ -234,12 +232,6 @@ impl<'a, 'cfg> Context<'a, 'cfg> {
 
             map.insert(crate_type.to_string(), Some((prefix.to_string(), suffix.to_string())));
         }
- 
-        // Manually handle the metadata case. If it is not supported by the
-        // compiler we'll error out elsewhere.
-        if crate_types.contains("metadata") {
-            map.insert("metadata".to_string(), Some(("lib".to_owned(), ".rmeta".to_owned())));
-        }
 
         let cfg = if has_cfg {
             Some(try!(lines.map(Cfg::from_str).collect()))
@@ -464,7 +456,8 @@ impl<'a, 'cfg> Context<'a, 'cfg> {
         // we don't want to link it up.
         if src_dir.ends_with("deps") {
             // Don't lift up library dependencies
-            if self.ws.members().find(|&p| p != unit.pkg).is_some() && !unit.target.is_bin() {
+            if self.ws.members().find(|&p| p == unit.pkg).is_none() &&
+               !unit.target.is_bin() {
                 None
             } else {
                 Some((
@@ -502,46 +495,53 @@ impl<'a, 'cfg> Context<'a, 'cfg> {
         let mut ret = Vec::new();
         let mut unsupported = Vec::new();
         {
-            let mut add = |crate_type: &str, linkable: bool| -> CargoResult<()> {
-                let crate_type = if crate_type == "lib" {"rlib"} else {crate_type};
-                match info.crate_types.get(crate_type) {
-                    Some(&Some((ref prefix, ref suffix))) => {
-                        let filename = out_dir.join(format!("{}{}{}", prefix, stem, suffix));
-                        let link_dst = link_stem.clone().map(|(ld, ls)| {
-                            ld.join(format!("{}{}{}", prefix, ls, suffix))
-                        });
-                        ret.push((filename, link_dst, linkable));
-                        Ok(())
-                    }
-                    // not supported, don't worry about it
-                    Some(&None) => {
-                        unsupported.push(crate_type.to_string());
-                        Ok(())
-                    }
-                    None => {
-                        bail!("failed to learn about crate-type `{}` early on",
-                              crate_type)
-                    }
-                }
-            };
-
             if unit.profile.check {
-                add("metadata", true)?;
+                let filename = out_dir.join(format!("lib{}.rmeta", stem));
+                let link_dst = link_stem.clone().map(|(ld, ls)| {
+                    ld.join(format!("lib{}.rmeta", ls))
+                });
+                ret.push((filename, link_dst, true));
             } else {
+                let mut add = |crate_type: &str, linkable: bool| -> CargoResult<()> {
+                    let crate_type = if crate_type == "lib" {"rlib"} else {crate_type};
+                    match info.crate_types.get(crate_type) {
+                        Some(&Some((ref prefix, ref suffix))) => {
+                            let filename = out_dir.join(format!("{}{}{}", prefix, stem, suffix));
+                            let link_dst = link_stem.clone().map(|(ld, ls)| {
+                                ld.join(format!("{}{}{}", prefix, ls, suffix))
+                            });
+                            ret.push((filename, link_dst, linkable));
+                            Ok(())
+                        }
+                        // not supported, don't worry about it
+                        Some(&None) => {
+                            unsupported.push(crate_type.to_string());
+                            Ok(())
+                        }
+                        None => {
+                            bail!("failed to learn about crate-type `{}` early on",
+                                  crate_type)
+                        }
+                    }
+                };
+
                 match *unit.target.kind() {
-                    TargetKind::Example |
                     TargetKind::Bin |
                     TargetKind::CustomBuild |
+                    TargetKind::ExampleBin |
                     TargetKind::Bench |
                     TargetKind::Test => {
                         add("bin", false)?;
                     }
-                    TargetKind::Lib(..) if unit.profile.test => {
+                    TargetKind::Lib(..) |
+                    TargetKind::ExampleLib(..)
+                    if unit.profile.test => {
                         add("bin", false)?;
                     }
-                    TargetKind::Lib(ref libs) => {
-                        for lib in libs {
-                            add(lib.crate_type(), lib.linkable())?;
+                    TargetKind::ExampleLib(ref kinds) |
+                    TargetKind::Lib(ref kinds) => {
+                        for kind in kinds {
+                            add(kind.crate_type(), kind.linkable())?;
                         }
                     }
                 }
@@ -566,7 +566,7 @@ impl<'a, 'cfg> Context<'a, 'cfg> {
     pub fn dep_targets(&self, unit: &Unit<'a>) -> CargoResult<Vec<Unit<'a>>> {
         if unit.profile.run_custom_build {
             return self.dep_run_custom_build(unit)
-        } else if unit.profile.doc {
+        } else if unit.profile.doc && !unit.profile.test {
             return self.doc_deps(unit);
         }
 
@@ -613,17 +613,10 @@ impl<'a, 'cfg> Context<'a, 'cfg> {
             match self.get_package(id) {
                 Ok(pkg) => {
                     pkg.targets().iter().find(|t| t.is_lib()).map(|t| {
-                        let profile = if unit.profile.check &&
-                                         !t.is_custom_build()
-                                         && !t.for_host() {
-                            &self.profiles.check
-                        } else {
-                            self.lib_profile()
-                        };
                         let unit = Unit {
                             pkg: pkg,
                             target: t,
-                            profile: profile,
+                            profile: self.lib_or_check_profile(unit, t),
                             kind: unit.kind.for_target(t),
                         };
                         Ok(unit)
@@ -645,7 +638,7 @@ impl<'a, 'cfg> Context<'a, 'cfg> {
         // the library of the same package. The call to `resolve.deps` above
         // didn't include `pkg` in the return values, so we need to special case
         // it here and see if we need to push `(pkg, pkg_lib_target)`.
-        if unit.target.is_lib() {
+        if unit.target.is_lib() && !unit.profile.doc {
             return Ok(ret)
         }
         ret.extend(self.maybe_lib(unit));
@@ -780,7 +773,7 @@ impl<'a, 'cfg> Context<'a, 'cfg> {
             Unit {
                 pkg: unit.pkg,
                 target: t,
-                profile: self.lib_profile(),
+                profile: self.lib_or_check_profile(unit, t),
                 kind: unit.kind.for_target(t),
             }
         })
@@ -848,12 +841,28 @@ impl<'a, 'cfg> Context<'a, 'cfg> {
         }
     }
 
+    pub fn lib_or_check_profile(&self, unit: &Unit, target: &Target) -> &'a Profile {
+        if unit.profile.check && !target.is_custom_build() && !target.for_host() {
+            &self.profiles.check
+        } else {
+            self.lib_profile()
+        }
+    }
+
     pub fn build_script_profile(&self, _pkg: &PackageId) -> &'a Profile {
         // TODO: should build scripts always be built with the same library
         //       profile? How is this controlled at the CLI layer?
         self.lib_profile()
     }
 
+    pub fn incremental_args(&self, unit: &Unit) -> CargoResult<Vec<String>> {
+        if self.incremental_enabled {
+            Ok(vec![format!("-Zincremental={}", self.layout(unit.kind).incremental().display())])
+        } else {
+            Ok(vec![])
+        }
+    }
+
     pub fn rustflags_args(&self, unit: &Unit) -> CargoResult<Vec<String>> {
         env_args(self.config, &self.build_config, unit.kind, "RUSTFLAGS")
     }
