@@ -11,12 +11,15 @@ use rustc_serialize::json;
 use core::{Package, PackageId, PackageSet, Target, Resolve};
 use core::{Profile, Profiles, Workspace};
 use core::shell::ColorConfig;
-use util::{self, CargoResult, ProcessBuilder, human, machine_message};
+use util::{self, CargoResult, ProcessBuilder, ProcessError, human, machine_message};
 use util::{Config, internal, ChainError, profile, join_paths, short_hash};
+use util::Freshness;
 
 use self::job::{Job, Work};
 use self::job_queue::JobQueue;
 
+use self::output_depinfo::output_depinfo;
+
 pub use self::compilation::Compilation;
 pub use self::context::{Context, Unit};
 pub use self::custom_build::{BuildOutput, BuildMap, BuildScripts};
@@ -29,6 +32,7 @@ mod job;
 mod job_queue;
 mod layout;
 mod links;
+mod output_depinfo;
 
 #[derive(PartialEq, Eq, Hash, Debug, Clone, Copy, PartialOrd, Ord)]
 pub enum Kind { Host, Target }
@@ -55,6 +59,36 @@ pub struct TargetConfig {
 
 pub type PackagesToBuild<'a> = [(&'a Package, Vec<(&'a Target, &'a Profile)>)];
 
+/// A glorified callback for executing calls to rustc. Rather than calling rustc
+/// directly, we'll use an Executor, giving clients an opportunity to intercept
+/// the build calls.
+pub trait Executor: Send + Sync + 'static {
+    fn init(&self, _cx: &Context) {}
+    /// If execution succeeds, the ContinueBuild value indicates whether Cargo
+    /// should continue with the build process for this package.
+    fn exec(&self, cmd: ProcessBuilder, _id: &PackageId) -> Result<(), ProcessError> {
+        cmd.exec()?;
+        Ok(())
+    }
+
+    fn exec_json(&self,
+                 cmd: ProcessBuilder,
+                 _id: &PackageId,
+                 handle_stdout: &mut FnMut(&str) -> CargoResult<()>,
+                 handle_srderr: &mut FnMut(&str) -> CargoResult<()>)
+                 -> Result<(), ProcessError> {
+        cmd.exec_with_streaming(handle_stdout, handle_srderr)?;
+        Ok(())
+    }
+}
+
+/// A DefaultExecutor calls rustc without doing anything else. It is Cargo's
+/// default behaviour.
+#[derive(Copy, Clone)]
+pub struct DefaultExecutor;
+
+impl Executor for DefaultExecutor {}
+
 // Returns a mapping of the root package plus its immediate dependencies to
 // where the compiled libraries are all located.
 pub fn compile_targets<'a, 'cfg: 'a>(ws: &Workspace<'cfg>,
@@ -63,7 +97,8 @@ pub fn compile_targets<'a, 'cfg: 'a>(ws: &Workspace<'cfg>,
                                      resolve: &'a Resolve,
                                      config: &'cfg Config,
                                      build_config: BuildConfig,
-                                     profiles: &'a Profiles)
+                                     profiles: &'a Profiles,
+                                     exec: Arc<Executor>)
                                      -> CargoResult<Compilation<'cfg>> {
     let units = pkg_targets.iter().flat_map(|&(pkg, ref targets)| {
         let default_kind = if build_config.requested_target.is_some() {
@@ -97,7 +132,7 @@ pub fn compile_targets<'a, 'cfg: 'a>(ws: &Workspace<'cfg>,
         // part of this, that's all done next as part of the `execute`
         // function which will run everything in order with proper
         // parallelism.
-        compile(&mut cx, &mut queue, unit)?;
+        compile(&mut cx, &mut queue, unit, exec.clone())?;
     }
 
     // Now that we've figured out everything that we're going to do, do it!
@@ -118,33 +153,31 @@ pub fn compile_targets<'a, 'cfg: 'a>(ws: &Workspace<'cfg>,
                 cx.compilation.binaries.push(bindst);
             } else if unit.target.is_lib() {
                 let pkgid = unit.pkg.package_id().clone();
-                cx.compilation.libraries.entry(pkgid).or_insert(Vec::new())
-                  .push((unit.target.clone(), dst));
+                cx.compilation.libraries.entry(pkgid).or_insert(HashSet::new())
+                  .insert((unit.target.clone(), dst));
             }
+        }
+
+        for dep in cx.dep_targets(unit)?.iter() {
             if !unit.target.is_lib() { continue }
 
-            // Include immediate lib deps as well
-            for unit in cx.dep_targets(unit)?.iter() {
-                if unit.profile.run_custom_build {
-                    let out_dir = cx.build_script_out_dir(unit).display().to_string();
-                    cx.compilation.extra_env.entry(unit.pkg.package_id().clone())
-                      .or_insert(Vec::new())
-                      .push(("OUT_DIR".to_string(), out_dir));
-                }
-
-                let pkgid = unit.pkg.package_id();
-                if !unit.target.is_lib() { continue }
-                if unit.profile.doc { continue }
-                if cx.compilation.libraries.contains_key(&pkgid) {
-                    continue
-                }
-
-                let v = cx.target_filenames(unit)?;
-                let v = v.into_iter().map(|(f, _, _)| {
-                    (unit.target.clone(), f)
-                }).collect::<Vec<_>>();
-                cx.compilation.libraries.insert(pkgid.clone(), v);
+            if dep.profile.run_custom_build {
+                let out_dir = cx.build_script_out_dir(dep).display().to_string();
+                cx.compilation.extra_env.entry(dep.pkg.package_id().clone())
+                  .or_insert(Vec::new())
+                  .push(("OUT_DIR".to_string(), out_dir));
             }
+
+            if !dep.target.is_lib() { continue }
+            if dep.profile.doc { continue }
+
+            let v = cx.target_filenames(dep)?;
+            cx.compilation.libraries
+                .entry(unit.pkg.package_id().clone())
+                .or_insert(HashSet::new())
+                .extend(v.into_iter().map(|(f, _, _)| {
+                    (dep.target.clone(), f)
+                }));
         }
 
         if let Some(feats) = cx.resolve.features(&unit.pkg.package_id()) {
@@ -152,6 +185,8 @@ pub fn compile_targets<'a, 'cfg: 'a>(ws: &Workspace<'cfg>,
                 .or_insert(HashSet::new())
                 .extend(feats.iter().map(|feat| format!("feature=\"{}\"", feat)));
         }
+
+        output_depinfo(&mut cx, unit)?;
     }
 
     for (&(ref pkg, _), output) in cx.build_state.outputs.lock().unwrap().iter() {
@@ -169,7 +204,8 @@ pub fn compile_targets<'a, 'cfg: 'a>(ws: &Workspace<'cfg>,
 
 fn compile<'a, 'cfg: 'a>(cx: &mut Context<'a, 'cfg>,
                          jobs: &mut JobQueue<'a>,
-                         unit: &Unit<'a>) -> CargoResult<()> {
+                         unit: &Unit<'a>,
+                         exec: Arc<Executor>) -> CargoResult<()> {
     if !cx.compiled.insert(*unit) {
         return Ok(())
     }
@@ -183,12 +219,15 @@ fn compile<'a, 'cfg: 'a>(cx: &mut Context<'a, 'cfg>,
 
     let (dirty, fresh, freshness) = if unit.profile.run_custom_build {
         custom_build::prepare(cx, unit)?
+    } else if unit.profile.doc && unit.profile.test {
+        // we run these targets later, so this is just a noop for now
+        (Work::new(|_| Ok(())), Work::new(|_| Ok(())), Freshness::Fresh)
     } else {
         let (freshness, dirty, fresh) = fingerprint::prepare_target(cx, unit)?;
         let work = if unit.profile.doc {
             rustdoc(cx, unit)?
         } else {
-            rustc(cx, unit)?
+            rustc(cx, unit, exec.clone())?
         };
         let link_work1 = link_targets(cx, unit)?;
         let link_work2 = link_targets(cx, unit)?;
@@ -202,13 +241,13 @@ fn compile<'a, 'cfg: 'a>(cx: &mut Context<'a, 'cfg>,
 
     // Be sure to compile all dependencies of this target as well.
     for unit in cx.dep_targets(unit)?.iter() {
-        compile(cx, jobs, unit)?;
+        compile(cx, jobs, unit, exec.clone())?;
     }
 
     Ok(())
 }
 
-fn rustc(cx: &mut Context, unit: &Unit) -> CargoResult<Work> {
+fn rustc(cx: &mut Context, unit: &Unit, exec: Arc<Executor>) -> CargoResult<Work> {
     let crate_types = unit.target.rustc_crate_types();
     let mut rustc = prepare_rustc(cx, crate_types, unit)?;
 
@@ -220,7 +259,6 @@ fn rustc(cx: &mut Context, unit: &Unit) -> CargoResult<Work> {
             rustc.arg("-Awarnings");
         }
     }
-    let has_custom_args = unit.profile.rustc_args.is_some();
 
     let filenames = cx.target_filenames(unit)?;
     let root = cx.out_dir(unit);
@@ -247,6 +285,7 @@ fn rustc(cx: &mut Context, unit: &Unit) -> CargoResult<Work> {
     let dep_info_loc = fingerprint::dep_info_loc(cx, unit);
     let cwd = cx.config.cwd().to_path_buf();
 
+    rustc.args(&cx.incremental_args(unit)?);
     rustc.args(&cx.rustflags_args(unit)?);
     let json_messages = cx.build_config.json_messages;
     let package_id = unit.pkg.package_id().clone();
@@ -257,6 +296,10 @@ fn rustc(cx: &mut Context, unit: &Unit) -> CargoResult<Work> {
                      .flat_map(|i| i)
                      .map(|s| s.to_string())
                      .collect::<Vec<_>>();
+
+    exec.init(cx);
+    let exec = exec.clone();
+
     return Ok(Work::new(move |state| {
         // Only at runtime have we discovered what the extra -L and -l
         // arguments are for native libraries, so we process those here. We
@@ -291,7 +334,7 @@ fn rustc(cx: &mut Context, unit: &Unit) -> CargoResult<Work> {
 
         state.running(&rustc);
         if json_messages {
-            rustc.exec_with_streaming(
+            exec.exec_json(rustc, &package_id,
                 &mut |line| if !line.is_empty() {
                     Err(internal(&format!("compiler stdout is not empty: `{}`", line)))
                 } else {
@@ -315,27 +358,29 @@ fn rustc(cx: &mut Context, unit: &Unit) -> CargoResult<Work> {
                         writeln!(io::stderr(), "{}", line)?;
                     }
                     Ok(())
-                },
-            ).map(|_| ())
+                }
+            ).chain_error(|| {
+                human(format!("Could not compile `{}`.", name))
+            })?;
         } else {
-            rustc.exec()
-        }.chain_error(|| {
-            human(format!("Could not compile `{}`.", name))
-        })?;
+            exec.exec(rustc, &package_id).chain_error(|| {
+                human(format!("Could not compile `{}`.", name))
+            })?;
+        }
 
         if do_rename && real_name != crate_name {
             let dst = &filenames[0].0;
             let src = dst.with_file_name(dst.file_name().unwrap()
                                             .to_str().unwrap()
                                             .replace(&real_name, &crate_name));
-            if !has_custom_args || src.exists() {
+            if src.exists() {
                 fs::rename(&src, &dst).chain_error(|| {
                     internal(format!("could not rename crate {:?}", src))
                 })?;
             }
         }
 
-        if !has_custom_args || fs::metadata(&rustc_dep_info_loc).is_ok() {
+        if fs::metadata(&rustc_dep_info_loc).is_ok() {
             info!("Renaming dep_info {:?} to {:?}", rustc_dep_info_loc, dep_info_loc);
             fs::rename(&rustc_dep_info_loc, &dep_info_loc).chain_error(|| {
                 internal(format!("could not rename dep info: {:?}",
@@ -460,7 +505,6 @@ fn prepare_rustc(cx: &mut Context,
                  unit: &Unit) -> CargoResult<ProcessBuilder> {
     let mut base = cx.compilation.rustc_process(unit.pkg)?;
     build_base_args(cx, &mut base, unit, &crate_types);
-    build_plugin_args(&mut base, cx, unit);
     build_deps_args(&mut base, cx, unit)?;
     Ok(base)
 }
@@ -566,14 +610,18 @@ fn build_base_args(cx: &mut Context,
         cmd.arg("--error-format").arg("json");
     }
 
-    if check {
-        cmd.arg("--crate-type").arg("metadata");
-    } else if !test {
+    if !test {
         for crate_type in crate_types.iter() {
             cmd.arg("--crate-type").arg(crate_type);
         }
     }
 
+    if check {
+        cmd.arg("--emit=dep-info,metadata");
+    } else {
+        cmd.arg("--emit=dep-info,link");
+    }
+
     let prefer_dynamic = (unit.target.for_host() &&
                           !unit.target.is_custom_build()) ||
                          (crate_types.contains(&"dylib") &&
@@ -614,8 +662,8 @@ fn build_base_args(cx: &mut Context,
         }
     }
 
-    if debuginfo {
-        cmd.arg("-g");
+    if let Some(debuginfo) = debuginfo {
+        cmd.arg("-C").arg(format!("debuginfo={}", debuginfo));
     }
 
     if let Some(ref args) = *rustc_args {
@@ -653,10 +701,9 @@ fn build_base_args(cx: &mut Context,
     if rpath {
         cmd.arg("-C").arg("rpath");
     }
-}
 
+    cmd.arg("--out-dir").arg(&cx.out_dir(unit));
 
-fn build_plugin_args(cmd: &mut ProcessBuilder, cx: &mut Context, unit: &Unit) {
     fn opt(cmd: &mut ProcessBuilder, key: &str, prefix: &str,
            val: Option<&OsStr>)  {
         if let Some(val) = val {
@@ -666,9 +713,6 @@ fn build_plugin_args(cmd: &mut ProcessBuilder, cx: &mut Context, unit: &Unit) {
         }
     }
 
-    cmd.arg("--out-dir").arg(&cx.out_dir(unit));
-    cmd.arg("--emit=dep-info,link");
-
     if unit.kind == Kind::Target {
         opt(cmd, "--target", "", cx.requested_target().map(|s| s.as_ref()));
     }
@@ -677,6 +721,7 @@ fn build_plugin_args(cmd: &mut ProcessBuilder, cx: &mut Context, unit: &Unit) {
     opt(cmd, "-C", "linker=", cx.linker(unit.kind).map(|s| s.as_ref()));
 }
 
+
 fn build_deps_args(cmd: &mut ProcessBuilder, cx: &mut Context, unit: &Unit)
                    -> CargoResult<()> {
     cmd.arg("-L").arg(&{
