@@ -46,19 +46,13 @@ pub enum Freshness {
     Dirty,
 }
 
-/// A trait for discovering the dependencies of a piece of data.
-pub trait Dependency: Hash + Eq + Clone {
-    type Context;
-    fn dependencies(&self, cx: &Self::Context) -> Vec<Self>;
-}
-
 impl Freshness {
     pub fn combine(&self, other: Freshness) -> Freshness {
         match *self { Fresh => other, Dirty => Dirty }
     }
 }
 
-impl<K: Dependency, V> DependencyQueue<K, V> {
+impl<K: Hash + Eq + Clone, V> DependencyQueue<K, V> {
     /// Creates a new dependency queue with 0 packages.
     pub fn new() -> DependencyQueue<K, V> {
         DependencyQueue {
@@ -73,8 +67,11 @@ impl<K: Dependency, V> DependencyQueue<K, V> {
     ///
     /// It is assumed that any dependencies of this package will eventually also
     /// be added to the dependency queue.
-    pub fn queue(&mut self, cx: &K::Context, fresh: Freshness,
-                 key: K, value: V) -> &mut V {
+    pub fn queue(&mut self,
+                 fresh: Freshness,
+                 key: K,
+                 value: V,
+                 dependencies: &[K]) -> &mut V {
         let slot = match self.dep_map.entry(key.clone()) {
             Occupied(v) => return &mut v.into_mut().1,
             Vacant(v) => v,
@@ -85,9 +82,10 @@ impl<K: Dependency, V> DependencyQueue<K, V> {
         }
 
         let mut my_dependencies = HashSet::new();
-        for dep in key.dependencies(cx).into_iter() {
+        for dep in dependencies {
             assert!(my_dependencies.insert(dep.clone()));
-            let rev = self.reverse_dep_map.entry(dep).or_insert(HashSet::new());
+            let rev = self.reverse_dep_map.entry(dep.clone())
+                                          .or_insert(HashSet::new());
             assert!(rev.insert(key.clone()));
         }
         &mut slot.insert((my_dependencies, value)).1
@@ -99,7 +97,7 @@ impl<K: Dependency, V> DependencyQueue<K, V> {
     /// `None` is returned then no packages are ready to be built.
     pub fn dequeue(&mut self) -> Option<(Freshness, K, V)> {
         let key = match self.dep_map.iter()
-                                    .find(|&(_, &(ref deps, _))| deps.len() == 0)
+                                    .find(|&(_, &(ref deps, _))| deps.is_empty())
                                     .map(|(key, _)| key.clone()) {
             Some(key) => key,
             None => return None
@@ -110,6 +108,11 @@ impl<K: Dependency, V> DependencyQueue<K, V> {
         Some((fresh, key, data))
     }
 
+    /// Returns whether there are remaining packages to be built.
+    pub fn is_empty(&self) -> bool {
+        self.dep_map.is_empty() && self.pending.is_empty()
+    }
+
     /// Returns the number of remaining packages to be built.
     pub fn len(&self) -> usize {
         self.dep_map.len() + self.pending.len()
