@@ -29,6 +29,7 @@ pub struct Layout {
     examples: Vec<PathBuf>,
     tests: Vec<PathBuf>,
     benches: Vec<PathBuf>,
+
 }
 
 impl Layout {
@@ -245,6 +246,7 @@ pub struct TomlManifest {
     target: Option<HashMap<String, TomlPlatform>>,
     replace: Option<HashMap<String, TomlDependency>>,
     workspace: Option<TomlWorkspace>,
+    badges: Option<HashMap<String, HashMap<String, String>>>,
 }
 
 #[derive(RustcDecodable, Clone, Default)]
@@ -275,12 +277,18 @@ impl Decodable for TomlOptLevel {
     }
 }
 
+#[derive(RustcDecodable, Clone)]
+pub enum U32OrBool {
+    U32(u32),
+    Bool(bool),
+}
+
 #[derive(RustcDecodable, Clone, Default)]
 pub struct TomlProfile {
     opt_level: Option<TomlOptLevel>,
     lto: Option<bool>,
     codegen_units: Option<u32>,
-    debug: Option<bool>,
+    debug: Option<U32OrBool>,
     debug_assertions: Option<bool>,
     rpath: Option<bool>,
     panic: Option<String>,
@@ -310,6 +318,7 @@ pub struct TomlProject {
     documentation: Option<String>,
     readme: Option<String>,
     keywords: Option<Vec<String>>,
+    categories: Option<Vec<String>>,
     license: Option<String>,
     license_file: Option<String>,
     repository: Option<String>,
@@ -549,7 +558,8 @@ impl TomlManifest {
         let new_build = self.maybe_custom_build(&project.build, &layout.root, &mut warnings);
 
         // Get targets
-        let targets = normalize(&lib,
+        let targets = normalize(&layout.root,
+                                &lib,
                                 &bins,
                                 new_build,
                                 &examples,
@@ -626,8 +636,9 @@ impl TomlManifest {
                 let name = dep.name();
                 let prev = names_sources.insert(name, dep.source_id());
                 if prev.is_some() && prev != Some(dep.source_id()) {
-                    bail!("found duplicate dependency name {}, but all \
-                           dependencies must have a unique name", name);
+                    bail!("Dependency '{}' has different source paths depending on the build \
+                           target. Each dependency must have a single canonical source path \
+                           irrespective of build target.", name);
                 }
             }
         }
@@ -646,6 +657,8 @@ impl TomlManifest {
             license_file: project.license_file.clone(),
             repository: project.repository.clone(),
             keywords: project.keywords.clone().unwrap_or(Vec::new()),
+            categories: project.categories.clone().unwrap_or(Vec::new()),
+            badges: self.badges.clone().unwrap_or_else(HashMap::new),
         };
 
         let workspace_config = match (self.workspace.as_ref(),
@@ -1090,7 +1103,8 @@ impl fmt::Debug for PathValue {
     }
 }
 
-fn normalize(lib: &Option<TomlLibTarget>,
+fn normalize(package_root: &Path,
+             lib: &Option<TomlLibTarget>,
              bins: &[TomlBinTarget],
              custom_build: Option<PathBuf>,
              examples: &[TomlExampleTarget],
@@ -1110,7 +1124,7 @@ fn normalize(lib: &Option<TomlLibTarget>,
               });
     }
 
-    fn lib_target(dst: &mut Vec<Target>, l: &TomlLibTarget) {
+    let lib_target = |dst: &mut Vec<Target>, l: &TomlLibTarget| {
         let path = l.path.clone().unwrap_or(
             PathValue::Path(Path::new("src").join(&format!("{}.rs", l.name())))
         );
@@ -1124,71 +1138,80 @@ fn normalize(lib: &Option<TomlLibTarget>,
         };
 
         let mut target = Target::lib_target(&l.name(), crate_types,
-                                            &path.to_path());
+                                            package_root.join(path.to_path()));
         configure(l, &mut target);
         dst.push(target);
-    }
+    };
 
-    fn bin_targets(dst: &mut Vec<Target>, bins: &[TomlBinTarget],
-                   default: &mut FnMut(&TomlBinTarget) -> PathBuf) {
+    let bin_targets = |dst: &mut Vec<Target>, bins: &[TomlBinTarget],
+                       default: &mut FnMut(&TomlBinTarget) -> PathBuf| {
         for bin in bins.iter() {
             let path = bin.path.clone().unwrap_or_else(|| {
                 PathValue::Path(default(bin))
             });
-            let mut target = Target::bin_target(&bin.name(), &path.to_path());
+            let mut target = Target::bin_target(&bin.name(), package_root.join(path.to_path()));
             configure(bin, &mut target);
             dst.push(target);
         }
-    }
+    };
 
-    fn custom_build_target(dst: &mut Vec<Target>, cmd: &Path) {
+    let custom_build_target = |dst: &mut Vec<Target>, cmd: &Path| {
         let name = format!("build-script-{}",
                            cmd.file_stem().and_then(|s| s.to_str()).unwrap_or(""));
 
-        dst.push(Target::custom_build_target(&name, cmd));
-    }
+        dst.push(Target::custom_build_target(&name, package_root.join(cmd)));
+    };
 
-    fn example_targets(dst: &mut Vec<Target>,
-                       examples: &[TomlExampleTarget],
-                       default: &mut FnMut(&TomlExampleTarget) -> PathBuf) {
+    let example_targets = |dst: &mut Vec<Target>,
+                           examples: &[TomlExampleTarget],
+                           default: &mut FnMut(&TomlExampleTarget) -> PathBuf| {
         for ex in examples.iter() {
             let path = ex.path.clone().unwrap_or_else(|| {
                 PathValue::Path(default(ex))
             });
 
-            let mut target = Target::example_target(&ex.name(), &path.to_path());
+            let crate_types = match ex.crate_type {
+                Some(ref kinds) => kinds.iter().map(|s| LibKind::from_str(s)).collect(),
+                None => Vec::new()
+            };
+
+            let mut target = Target::example_target(
+                &ex.name(),
+                crate_types,
+                package_root.join(path.to_path())
+            );
             configure(ex, &mut target);
             dst.push(target);
         }
-    }
+    };
 
-    fn test_targets(dst: &mut Vec<Target>,
-                    tests: &[TomlTestTarget],
-                    default: &mut FnMut(&TomlTestTarget) -> PathBuf) {
+    let test_targets = |dst: &mut Vec<Target>,
+                        tests: &[TomlTestTarget],
+                        default: &mut FnMut(&TomlTestTarget) -> PathBuf| {
         for test in tests.iter() {
             let path = test.path.clone().unwrap_or_else(|| {
                 PathValue::Path(default(test))
             });
 
-            let mut target = Target::test_target(&test.name(), &path.to_path());
+            let mut target = Target::test_target(&test.name(), package_root.join(path.to_path()));
             configure(test, &mut target);
             dst.push(target);
         }
-    }
+    };
 
-    fn bench_targets(dst: &mut Vec<Target>,
-                     benches: &[TomlBenchTarget],
-                     default: &mut FnMut(&TomlBenchTarget) -> PathBuf) {
+    let bench_targets = |dst: &mut Vec<Target>,
+                         benches: &[TomlBenchTarget],
+                         default: &mut FnMut(&TomlBenchTarget) -> PathBuf| {
         for bench in benches.iter() {
             let path = bench.path.clone().unwrap_or_else(|| {
                 PathValue::Path(default(bench))
             });
 
-            let mut target = Target::bench_target(&bench.name(), &path.to_path());
+            let mut target = Target::bench_target(&bench.name(), package_root.join(path.to_path()));
             configure(bench, &mut target);
             dst.push(target);
         }
-    }
+    };
 
     let mut ret = Vec::new();
 
@@ -1250,6 +1273,7 @@ fn build_profiles(profiles: &Option<TomlProfiles>) -> Profiles {
         custom_build: Profile::default_custom_build(),
         check: merge(Profile::default_check(),
                      profiles.and_then(|p| p.dev.as_ref())),
+        doctest: Profile::default_doctest(),
     };
     // The test/bench targets cannot have panic=abort because they'll all get
     // compiled with --test which requires the unwind runtime currently
@@ -1261,12 +1285,18 @@ fn build_profiles(profiles: &Option<TomlProfiles>) -> Profiles {
 
     fn merge(profile: Profile, toml: Option<&TomlProfile>) -> Profile {
         let &TomlProfile {
-            ref opt_level, lto, codegen_units, debug, debug_assertions, rpath,
+            ref opt_level, lto, codegen_units, ref debug, debug_assertions, rpath,
             ref panic
         } = match toml {
             Some(toml) => toml,
             None => return profile,
         };
+        let debug = match *debug {
+            Some(U32OrBool::U32(debug)) => Some(Some(debug)),
+            Some(U32OrBool::Bool(true)) => Some(Some(2)),
+            Some(U32OrBool::Bool(false)) => Some(None),
+            None => None,
+        };
         Profile {
             opt_level: opt_level.clone().unwrap_or(TomlOptLevel(profile.opt_level)).0,
             lto: lto.unwrap_or(profile.lto),
