@@ -114,6 +114,25 @@ impl ProjectBuilder {
 
     pub fn url(&self) -> Url { path2url(self.root()) }
 
+    pub fn target_debug_dir(&self) -> PathBuf {
+        self.build_dir().join("debug")
+    }
+
+    pub fn example_lib(&self, name: &str, kind: &str) -> PathBuf {
+        let prefix = ProjectBuilder::get_lib_prefix(kind);
+
+        let extension = ProjectBuilder::get_lib_extension(kind);
+
+        let lib_file_name = format!("{}{}.{}",
+                                    prefix,
+                                    name,
+                                    extension);
+
+        self.target_debug_dir()
+            .join("examples")
+            .join(&lib_file_name)
+    }
+
     pub fn bin(&self, b: &str) -> PathBuf {
         self.build_dir().join("debug").join(&format!("{}{}", b,
                                                      env::consts::EXE_SUFFIX))
@@ -188,6 +207,43 @@ impl ProjectBuilder {
         buffer
     }
 
+    fn get_lib_prefix(kind: &str) -> &str {
+        match kind {
+            "lib" | "rlib" => "lib",
+            "staticlib" | "dylib" | "proc-macro" => {
+                if cfg!(windows) {
+                    ""
+                } else {
+                    "lib"
+                }
+            }
+            _ => unreachable!()
+        }
+    }
+
+    fn get_lib_extension(kind: &str) -> &str {
+        match kind {
+            "lib" | "rlib" => "rlib",
+            "staticlib" => {
+                if cfg!(windows) {
+                    "lib"
+                } else {
+                    "a"
+                }
+            }
+            "dylib" | "proc-macro" => {
+                if cfg!(windows) {
+                    "dll"
+                } else if cfg!(target_os="macos") {
+                    "dylib"
+                } else {
+                    "so"
+                }
+            }
+            _ => unreachable!()
+        }
+    }
+
     fn rm_root(&self) {
         self.root.rm_rf()
     }
@@ -263,6 +319,8 @@ pub struct Execs {
     expect_exit_code: Option<i32>,
     expect_stdout_contains: Vec<String>,
     expect_stderr_contains: Vec<String>,
+    expect_stdout_not_contains: Vec<String>,
+    expect_stderr_not_contains: Vec<String>,
     expect_json: Option<Vec<Json>>,
 }
 
@@ -292,6 +350,16 @@ impl Execs {
         self
     }
 
+    pub fn with_stdout_does_not_contain<S: ToString>(mut self, expected: S) -> Execs {
+        self.expect_stdout_not_contains.push(expected.to_string());
+        self
+    }
+
+    pub fn with_stderr_does_not_contain<S: ToString>(mut self, expected: S) -> Execs {
+        self.expect_stderr_not_contains.push(expected.to_string());
+        self
+    }
+
     pub fn with_json(mut self, expected: &str) -> Execs {
         self.expect_json = Some(expected.split("\n\n").map(|obj| {
             Json::from_str(obj).unwrap()
@@ -321,14 +389,22 @@ impl Execs {
 
     fn match_stdout(&self, actual: &Output) -> ham::MatchResult {
         self.match_std(self.expect_stdout.as_ref(), &actual.stdout,
-                       "stdout", &actual.stderr, false)?;
+                       "stdout", &actual.stderr, MatchKind::Exact)?;
         for expect in self.expect_stdout_contains.iter() {
             self.match_std(Some(expect), &actual.stdout, "stdout",
-                           &actual.stderr, true)?;
+                           &actual.stderr, MatchKind::Partial)?;
         }
         for expect in self.expect_stderr_contains.iter() {
             self.match_std(Some(expect), &actual.stderr, "stderr",
-                           &actual.stdout, true)?;
+                           &actual.stdout, MatchKind::Partial)?;
+        }
+        for expect in self.expect_stdout_not_contains.iter() {
+            self.match_std(Some(expect), &actual.stdout, "stdout",
+                           &actual.stderr, MatchKind::NotPresent)?;
+        }
+        for expect in self.expect_stderr_not_contains.iter() {
+            self.match_std(Some(expect), &actual.stderr, "stderr",
+                           &actual.stdout, MatchKind::NotPresent)?;
         }
 
         if let Some(ref objects) = self.expect_json {
@@ -349,12 +425,12 @@ impl Execs {
 
     fn match_stderr(&self, actual: &Output) -> ham::MatchResult {
         self.match_std(self.expect_stderr.as_ref(), &actual.stderr,
-                       "stderr", &actual.stdout, false)
+                       "stderr", &actual.stdout, MatchKind::Exact)
     }
 
     fn match_std(&self, expected: Option<&String>, actual: &[u8],
                  description: &str, extra: &[u8],
-                 partial: bool) -> ham::MatchResult {
+                 kind: MatchKind) -> ham::MatchResult {
         let out = match expected {
             Some(out) => out,
             None => return ham::success(),
@@ -368,33 +444,46 @@ impl Execs {
         let actual = actual.replace("\r", "");
         let actual = actual.replace("\t", "<tab>");
 
-        let mut a = actual.lines();
-        let e = out.lines();
+        match kind {
+            MatchKind::Exact => {
+                let a = actual.lines();
+                let e = out.lines();
 
-        if partial {
-            let mut diffs = self.diff_lines(a.clone(), e.clone(), partial);
-            while let Some(..) = a.next() {
-                let a = self.diff_lines(a.clone(), e.clone(), partial);
-                if a.len() < diffs.len() {
-                    diffs = a;
-                }
+                let diffs = self.diff_lines(a, e, false);
+                ham::expect(diffs.is_empty(),
+                            format!("differences:\n\
+                                    {}\n\n\
+                                    other output:\n\
+                                    `{}`", diffs.join("\n"),
+                                    String::from_utf8_lossy(extra)))
             }
-            ham::expect(diffs.is_empty(),
-                        format!("expected to find:\n\
-                                 {}\n\n\
-                                 did not find in output:\n\
-                                 {}", out,
-                                 actual))
-        } else {
-            let diffs = self.diff_lines(a, e, partial);
-            ham::expect(diffs.is_empty(),
-                        format!("differences:\n\
-                                {}\n\n\
-                                other output:\n\
-                                `{}`", diffs.join("\n"),
-                                String::from_utf8_lossy(extra)))
-        }
+            MatchKind::Partial => {
+                let mut a = actual.lines();
+                let e = out.lines();
 
+                let mut diffs = self.diff_lines(a.clone(), e.clone(), true);
+                while let Some(..) = a.next() {
+                    let a = self.diff_lines(a.clone(), e.clone(), true);
+                    if a.len() < diffs.len() {
+                        diffs = a;
+                    }
+                }
+                ham::expect(diffs.is_empty(),
+                            format!("expected to find:\n\
+                                     {}\n\n\
+                                     did not find in output:\n\
+                                     {}", out,
+                                     actual))
+            }
+            MatchKind::NotPresent => {
+                ham::expect(!actual.contains(out),
+                            format!("expected not to find:\n\
+                                     {}\n\n\
+                                     but found in output:\n\
+                                     {}", out,
+                                     actual))
+            }
+        }
     }
 
     fn match_json(&self, expected: &Json, line: &str) -> ham::MatchResult {
@@ -441,6 +530,13 @@ impl Execs {
     }
 }
 
+#[derive(Debug, PartialEq, Eq, Clone, Copy)]
+enum MatchKind {
+    Exact,
+    Partial,
+    NotPresent,
+}
+
 pub fn lines_match(expected: &str, mut actual: &str) -> bool {
     let expected = substitute_macros(expected);
     for (i, part) in expected.split("[..]").enumerate() {
@@ -474,6 +570,8 @@ fn lines_match_works() {
 
 // Compares JSON object for approximate equality.
 // You can use `[..]` wildcard in strings (useful for OS dependent things such as paths).
+// You can use a `"{...}"` string literal as a wildcard for arbitrary nested JSON (useful
+// for parts of object emitted by other programs (e.g. rustc) rather than Cargo itself).
 // Arrays are sorted before comparison.
 fn find_mismatch<'a>(expected: &'a Json, actual: &'a Json) -> Option<(&'a Json, &'a Json)> {
     use rustc_serialize::json::Json::*;
@@ -490,8 +588,7 @@ fn find_mismatch<'a>(expected: &'a Json, actual: &'a Json) -> Option<(&'a Json,
 
             fn sorted(xs: &Vec<Json>) -> Vec<&Json> {
                 let mut result = xs.iter().collect::<Vec<_>>();
-                // `unwrap` should be safe because JSON spec does not allow NaNs
-                result.sort_by(|x, y| x.partial_cmp(y).unwrap());
+                result.sort_by(|x, y| x.partial_cmp(y).expect("JSON spec does not allow NaNs"));
                 result
             }
 
@@ -510,6 +607,8 @@ fn find_mismatch<'a>(expected: &'a Json, actual: &'a Json) -> Option<(&'a Json,
              .nth(0)
         }
         (&Null, &Null) => None,
+        // magic string literal "{...}" acts as wildcard for any sub-JSON
+        (&String(ref l), _) if l == "{...}" => None,
         _ => Some((expected, actual)),
     }
 
@@ -589,6 +688,8 @@ pub fn execs() -> Execs {
         expect_exit_code: None,
         expect_stdout_contains: Vec::new(),
         expect_stderr_contains: Vec::new(),
+        expect_stdout_not_contains: Vec::new(),
+        expect_stderr_not_contains: Vec::new(),
         expect_json: None,
     }
 }
