@@ -40,7 +40,7 @@ fn simple() {
 [DOWNLOADING] bar v0.0.1 (registry file://[..])
 [COMPILING] bar v0.0.1
 [COMPILING] foo v0.0.1 ({dir})
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 ",
         dir = p.url(),
         reg = registry::registry())));
@@ -51,7 +51,7 @@ fn simple() {
 [UPDATING] registry `{reg}`
 [COMPILING] bar v0.0.1
 [COMPILING] foo v0.0.1 ({dir})
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 ",
         dir = p.url(),
         reg = registry::registry())));
@@ -82,7 +82,7 @@ fn deps() {
 [COMPILING] baz v0.0.1
 [COMPILING] bar v0.0.1
 [COMPILING] foo v0.0.1 ({dir})
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 ",
         dir = p.url(),
         reg = registry::registry())));
@@ -213,7 +213,7 @@ version required: >= 0.0.0
 [DOWNLOADING] notyet v0.0.1 (registry file://[..])
 [COMPILING] notyet v0.0.1
 [COMPILING] foo v0.0.1 ({dir})
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 ",
         dir = p.url(),
         reg = registry::registry())));
@@ -267,7 +267,7 @@ version required: ^0.0.1
 [DOWNLOADING] notyet v0.0.1 (registry file://[..])
 [COMPILING] notyet v0.0.1
 [COMPILING] foo v0.0.1 ({dir}[..])
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 ", dir = p.url())));
 }
 
@@ -294,7 +294,7 @@ fn lockfile_locks() {
 [DOWNLOADING] bar v0.0.1 (registry file://[..])
 [COMPILING] bar v0.0.1
 [COMPILING] foo v0.0.1 ({dir})
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 ",
    dir = p.url())));
 
@@ -331,7 +331,7 @@ fn lockfile_locks_transitively() {
 [COMPILING] baz v0.0.1
 [COMPILING] bar v0.0.1
 [COMPILING] foo v0.0.1 ({dir})
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 ",
    dir = p.url())));
 
@@ -371,7 +371,7 @@ fn yanks_are_not_used() {
 [COMPILING] baz v0.0.1
 [COMPILING] bar v0.0.1
 [COMPILING] foo v0.0.1 ({dir})
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 ",
    dir = p.url())));
 }
@@ -464,7 +464,7 @@ fn update_with_lockfile_if_packages_missing() {
                 execs().with_status(0).with_stderr("\
 [UPDATING] registry `[..]`
 [DOWNLOADING] bar v0.0.1 (registry file://[..])
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 "));
 }
 
@@ -505,7 +505,7 @@ fn update_lockfile() {
 [DOWNLOADING] [..] v0.0.2 (registry file://[..])
 [COMPILING] bar v0.0.2
 [COMPILING] foo v0.0.1 ({dir})
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 ",
    dir = p.url())));
 
@@ -523,7 +523,7 @@ fn update_lockfile() {
 [DOWNLOADING] [..] v0.0.3 (registry file://[..])
 [COMPILING] bar v0.0.3
 [COMPILING] foo v0.0.1 ({dir})
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 ",
    dir = p.url())));
 
@@ -573,7 +573,7 @@ fn dev_dependency_not_used() {
 [DOWNLOADING] [..] v0.0.1 (registry file://[..])
 [COMPILING] bar v0.0.1
 [COMPILING] foo v0.0.1 ({dir})
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 ",
    dir = p.url())));
 }
@@ -657,7 +657,7 @@ fn updating_a_dep() {
 [COMPILING] bar v0.0.1
 [COMPILING] a v0.0.1 ({dir}/a)
 [COMPILING] foo v0.0.1 ({dir})
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 ",
    dir = p.url())));
 
@@ -680,7 +680,7 @@ fn updating_a_dep() {
 [COMPILING] bar v0.1.0
 [COMPILING] a v0.0.1 ({dir}/a)
 [COMPILING] foo v0.0.1 ({dir})
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 ",
    dir = p.url())));
 }
@@ -726,7 +726,7 @@ fn git_and_registry_dep() {
 [COMPILING] a v0.0.1
 [COMPILING] b v0.0.1 ([..])
 [COMPILING] foo v0.0.1 ({dir})
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 ",
    dir = p.url())));
     p.root().move_into_the_past();
@@ -791,7 +791,7 @@ fn update_publish_then_update() {
 [DOWNLOADING] a v0.1.1 (registry file://[..])
 [COMPILING] a v0.1.1
 [COMPILING] foo v0.5.0 ({dir})
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 ",
    dir = p.url())));
 
@@ -859,7 +859,7 @@ fn update_transitive_dependency() {
 [COMPILING] b v0.1.1
 [COMPILING] a v0.1.0
 [COMPILING] foo v0.5.0 ([..])
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 "));
 }
 
@@ -1076,7 +1076,7 @@ fn only_download_relevant() {
 [DOWNLOADING] baz v0.1.0 ([..])
 [COMPILING] baz v0.1.0
 [COMPILING] bar v0.5.0 ([..])
-[FINISHED] debug [unoptimized + debuginfo] target(s) in [..] secs
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..] secs
 "));
 }
 
